#!/bin/bash

# Copyright (C) 2012-2018 Marc Ferland
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

BIN=`which cppcheck`
OPT="--enable=all --language=c --platform=unix64 --std=c99"
FILES="yazc/ lib/"

if [ ! -x "$BIN" ]; then
    echo >&2 "cppcheck is not installed."
    exit 1
fi

${BIN} ${OPT} ${FILES}
